@@ -5,7 +5,7 @@
  * Copyright © 2004-2019 Tom St Denis
  * Copyright © 2004 g10 Code GmbH
  * Copyright © 2002-2015 Wei Dai & Igor Pavlov
- * Copyright © 2015-2024 Pete Batard <pete@akeo.ie>
+ * Copyright © 2015-2025 Pete Batard <pete@akeo.ie>
  * Copyright © 2022 Jeffrey Walton <noloader@gmail.com>
  * Copyright © 2016 Alexander Graf
  *
@@ -65,11 +65,12 @@
 #include <string.h>
 #include <inttypes.h>
 #include <errno.h>
+#include <intrin.h>
 #include <windows.h>
 #include <windowsx.h>
 
 #include "db.h"
-#include "cpu.h"
+#include "efi.h"
 #include "rufus.h"
 #include "winio.h"
 #include "missing.h"
@@ -77,14 +78,10 @@
 #include "msapi_utf8.h"
 #include "localization.h"
 
-/* Includes for SHA-1 and SHA-256 intrinsics */
-#if defined(CPU_X86_SHA1_ACCELERATION) || defined(CPU_X86_SHA256_ACCELERATION)
-#if defined(_MSC_VER)
-#include <immintrin.h>
-#elif defined(__GNUC__)
-#include <stdint.h>
-#include <x86intrin.h>
-#endif
+#if (defined(_M_IX86) || defined(_M_X64) || defined(__i386__) || defined(__i386) || \
+     defined(_X86_) || defined(__I86__) || defined(__x86_64__))
+#define CPU_X86_SHA1_ACCELERATION       1
+#define CPU_X86_SHA256_ACCELERATION     1
 #endif
 
 #if defined(_MSC_VER)
@@ -107,6 +104,7 @@ char hash_str[HASH_MAX][150];
 HANDLE data_ready[HASH_MAX] = { 0 }, thread_ready[HASH_MAX] = { 0 };
 DWORD read_size[NUM_BUFFERS];
 BOOL enable_extra_hashes = FALSE, validate_md5sum = FALSE;
+BOOL cpu_has_sha1_accel = FALSE, cpu_has_sha256_accel = FALSE;
 uint8_t ALIGNED(64) buffer[NUM_BUFFERS][BUFFER_SIZE];
 uint8_t* pe256ssp = NULL;
 uint32_t proc_bufnum, hash_count[HASH_MAX] = { MD5_HASHSIZE, SHA1_HASHSIZE, SHA256_HASHSIZE, SHA512_HASHSIZE };
@@ -116,9 +114,81 @@ StrArray modified_files = { 0 };
 
 extern int default_thread_priority;
 extern const char* efi_archname[ARCH_MAX];
-extern char* sbat_level_txt;
+extern char *sbat_level_txt, *sb_active_txt, *sb_revoked_txt;
 extern BOOL expert_mode, usb_debug;
 
+/*
+ * Detect if the processor supports SHA-1 acceleration. We only check for
+ * the three ISAs we need - SSSE3, SSE4.1 and SHA. We don't check for OS
+ * support or XSAVE because that's been enabled since Windows 2000.
+ */
+BOOL DetectSHA1Acceleration(void)
+{
+#if defined(CPU_X86_SHA1_ACCELERATION)
+#if defined(_MSC_VER)
+	uint32_t regs0[4] = { 0,0,0,0 }, regs1[4] = { 0,0,0,0 }, regs7[4] = { 0,0,0,0 };
+	const uint32_t SSSE3_BIT = 1u << 9; /* Function 1, Bit  9 of ECX */
+	const uint32_t SSE41_BIT = 1u << 19; /* Function 1, Bit 19 of ECX */
+	const uint32_t SHA_BIT = 1u << 29; /* Function 7, Bit 29 of EBX */
+
+	__cpuid(regs0, 0);
+	const uint32_t highest = regs0[0]; /*EAX*/
+
+	if (highest >= 0x01) {
+		__cpuidex(regs1, 1, 0);
+	}
+	if (highest >= 0x07) {
+		__cpuidex(regs7, 7, 0);
+	}
+
+	return (regs1[2] /*ECX*/ & SSSE3_BIT) && (regs1[2] /*ECX*/ & SSE41_BIT) && (regs7[1] /*EBX*/ & SHA_BIT) ? TRUE : FALSE;
+#elif defined(__GNUC__) || defined(__clang__)
+	/* __builtin_cpu_supports available in GCC 4.8.1 and above */
+	return __builtin_cpu_supports("ssse3") && __builtin_cpu_supports("sse4.1") && __builtin_cpu_supports("sha") ? TRUE : FALSE;
+#else
+	return FALSE;
+#endif
+#else
+	return FALSE;
+#endif
+}
+
+/*
+ * Detect if the processor supports SHA-256 acceleration. We only check for
+ * the three ISAs we need - SSSE3, SSE4.1 and SHA. We don't check for OS
+ * support or XSAVE because that's been enabled since Windows 2000.
+ */
+BOOL DetectSHA256Acceleration(void)
+{
+#if defined(CPU_X86_SHA256_ACCELERATION)
+#if defined(_MSC_VER)
+	uint32_t regs0[4] = { 0,0,0,0 }, regs1[4] = { 0,0,0,0 }, regs7[4] = { 0,0,0,0 };
+	const uint32_t SSSE3_BIT = 1u << 9; /* Function 1, Bit  9 of ECX */
+	const uint32_t SSE41_BIT = 1u << 19; /* Function 1, Bit 19 of ECX */
+	const uint32_t SHA_BIT = 1u << 29; /* Function 7, Bit 29 of EBX */
+
+	__cpuid(regs0, 0);
+	const uint32_t highest = regs0[0]; /*EAX*/
+
+	if (highest >= 0x01) {
+		__cpuidex(regs1, 1, 0);
+	}
+	if (highest >= 0x07) {
+		__cpuidex(regs7, 7, 0);
+	}
+
+	return (regs1[2] /*ECX*/ & SSSE3_BIT) && (regs1[2] /*ECX*/ & SSE41_BIT) && (regs7[1] /*EBX*/ & SHA_BIT) ? TRUE : FALSE;
+#elif defined(__GNUC__) || defined(__clang__)
+	/* __builtin_cpu_supports available in GCC 4.8.1 and above */
+	return __builtin_cpu_supports("ssse3") && __builtin_cpu_supports("sse4.1") && __builtin_cpu_supports("sha") ? TRUE : FALSE;
+#else
+	return FALSE;
+#endif
+#else
+	return FALSE;
+#endif
+}
+
 /*
  * Rotate 32 or 64 bit integers by n bytes.
  * Don't bother trying to hand-optimize those, as the
@@ -1627,7 +1697,7 @@ BOOL efi_image_parse(uint8_t* efi, size_t len, struct efi_image_regions** regp)
 	if (len < 0x80)
 		return FALSE;
 	dos = (void*)efi;
-	if (dos->e_lfanew > len - 0x40)
+	if (dos->e_lfanew > (LONG)len - 0x40)
 		return FALSE;
 	nt = (void*)(efi + dos->e_lfanew);
 	authsz = 0;
@@ -1817,19 +1887,21 @@ out:
  */
 INT_PTR CALLBACK HashCallback(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
 {
+	static HFONT hFont = NULL;
 	int i, dw, dh;
 	RECT rc;
-	HFONT hFont;
 	HDC hDC;
 
 	switch (message) {
 	case WM_INITDIALOG:
 		apply_localization(IDD_HASH, hDlg);
-		hDC = GetDC(hDlg);
-		hFont = CreateFontA(-MulDiv(9, GetDeviceCaps(hDC, LOGPIXELSY), 72),
-			0, 0, 0, FW_NORMAL, FALSE, FALSE, FALSE, DEFAULT_CHARSET,
-			0, 0, PROOF_QUALITY, 0, "Courier New");
-		safe_release_dc(hDlg, hDC);
+		if (hFont == NULL) {
+			hDC = GetDC(hDlg);
+			hFont = CreateFontA(-MulDiv(9, GetDeviceCaps(hDC, LOGPIXELSY), 72),
+				0, 0, 0, FW_NORMAL, FALSE, FALSE, FALSE, DEFAULT_CHARSET,
+				0, 0, PROOF_QUALITY, 0, "Courier New");
+			safe_release_dc(hDlg, hDC);
+		}
 		SendDlgItemMessageA(hDlg, IDC_MD5, WM_SETFONT, (WPARAM)hFont, TRUE);
 		SendDlgItemMessageA(hDlg, IDC_SHA1, WM_SETFONT, (WPARAM)hFont, TRUE);
 		SendDlgItemMessageA(hDlg, IDC_SHA256, WM_SETFONT, (WPARAM)hFont, TRUE);
@@ -1873,6 +1945,9 @@ INT_PTR CALLBACK HashCallback(HWND hDlg, UINT message, WPARAM wParam, LPARAM lPa
 		SendMessage(hDlg, WM_NEXTDLGCTL, (WPARAM)GetDlgItem(hDlg, IDOK), TRUE);
 		CenterDialog(hDlg, NULL);
 		break;
+	case WM_NCDESTROY:
+		safe_delete_object(hFont);
+		break;
 	case WM_COMMAND:
 		switch (LOWORD(wParam)) {
 		case IDOK:
@@ -2087,6 +2162,12 @@ static BOOL IsRevokedBySbat(uint8_t* buf, uint32_t len)
 	uint32_t i, j, sbat_len;
 	sbat_entry_t entry;
 
+	// Fall back to embedded sbat_level.txt if we couldn't access remote
+	if (sbat_entries == NULL) {
+		sbat_level_txt = safe_strdup(db_sbat_level_txt);
+		sbat_entries = GetSbatEntries(sbat_level_txt);
+	}
+	assert(sbat_entries != NULL);
 	if (sbat_entries == NULL)
 		return FALSE;
 
@@ -2122,6 +2203,68 @@ static BOOL IsRevokedBySbat(uint8_t* buf, uint32_t len)
 	return FALSE;
 }
 
+// NB: Can be tested using en_windows_8_1_x64_dvd_2707217.iso
+extern BOOL UseLocalDbx(int arch);
+static BOOL IsRevokedByDbx(uint8_t* hash, uint8_t* buf, uint32_t len)
+{
+	EFI_VARIABLE_AUTHENTICATION_2* efi_var_auth;
+	EFI_SIGNATURE_LIST* efi_sig_list;
+	BYTE* dbx_data = NULL;
+	BOOL ret = FALSE, needs_free = FALSE;
+	DWORD dbx_size = 0;
+	char dbx_name[32], path[MAX_PATH];
+	uint32_t i, fluff_size, nb_entries;
+
+	i = MachineToArch(GetPeArch(buf));
+	if (i == ARCH_UNKNOWN)
+		goto out;
+
+	// Check if a more recent local DBX should be preferred over embedded
+	static_sprintf(dbx_name, "dbx_%s.bin", efi_archname[i]);
+	if (UseLocalDbx(i)) {
+		static_sprintf(path, "%s\\%s\\%s", app_data_dir, FILES_DIR, dbx_name);
+		dbx_size = read_file(path, &dbx_data);
+		needs_free = (dbx_data != NULL);
+		if (needs_free)
+			duprintf("  Using local %s for revocation check", path);
+	}
+	if (dbx_size == 0) {
+		dbx_data = (BYTE*)GetResource(hMainInstance, MAKEINTRESOURCEA(IDR_DBX + i),
+			_RT_RCDATA, dbx_name, &dbx_size, FALSE);
+	}
+	if (dbx_data == NULL || dbx_size <= sizeof(EFI_VARIABLE_AUTHENTICATION_2))
+		goto out;
+
+	efi_var_auth = (EFI_VARIABLE_AUTHENTICATION_2*)dbx_data;
+	fluff_size = efi_var_auth->AuthInfo.Hdr.dwLength + sizeof(EFI_TIME);
+	if (dbx_size <= fluff_size)
+		goto out;
+	efi_sig_list = (EFI_SIGNATURE_LIST*)&dbx_data[fluff_size];
+	fluff_size += sizeof(EFI_SIGNATURE_LIST);
+	if (dbx_size <= fluff_size)
+		goto out;
+	// Expect SHA-256 hashes
+	if (!CompareGUID(&efi_sig_list->SignatureType, &EFI_CERT_SHA256_GUID)) {
+		uprintf("  Warning: %s is not using SHA-256 hashes - Cannot check for UEFI revocation!", dbx_name);
+		goto out;
+	}
+	fluff_size += efi_sig_list->SignatureHeaderSize;
+	assert(efi_sig_list->SignatureSize != 0);
+	nb_entries = (efi_sig_list->SignatureListSize - efi_sig_list->SignatureHeaderSize - sizeof(EFI_SIGNATURE_LIST)) / efi_sig_list->SignatureSize;
+	assert(dbx_size >= fluff_size + nb_entries * efi_sig_list->SignatureSize);
+
+	fluff_size += sizeof(GUID);
+	for (i = 0; i < nb_entries && !ret; i++) {
+		if (memcmp(hash, &dbx_data[fluff_size + i * efi_sig_list->SignatureSize], SHA256_HASHSIZE) == 0)
+			ret = TRUE;
+	}
+
+out:
+	if (needs_free)
+		free(dbx_data);
+	return ret;
+}
+
 static BOOL IsRevokedBySvn(uint8_t* buf, uint32_t len)
 {
 	wchar_t* rsrc_name = NULL;
@@ -2164,7 +2307,7 @@ static BOOL IsRevokedBySvn(uint8_t* buf, uint32_t len)
 						return TRUE;
 				}
 			} else {
-				uprintf("WARNING: Unexpected Secure Version Number size");
+				uprintf("  Warning: Unexpected Secure Version Number size");
 			}
 		}
 	}
@@ -2173,13 +2316,24 @@ static BOOL IsRevokedBySvn(uint8_t* buf, uint32_t len)
 
 static BOOL IsRevokedByCert(cert_info_t* info)
 {
-	int i;
+	uint32_t i;
 
-	for (i = 0; i < ARRAYSIZE(certdbx); i += SHA1_HASHSIZE) {
-		if (!expert_mode)
-			continue;
-		if (memcmp(info->thumbprint, &certdbx[i], SHA1_HASHSIZE) == 0) {
-			uprintf("Found '%s' revoked certificate", info->name);
+	// TODO: Enable this for non expert mode after enforcement of PCA2011 cert revocation
+	if (!expert_mode)
+		return FALSE;
+
+	// Fall back to embedded Secure Boot thumbprints if we couldn't access remote
+	if (sb_revoked_certs == NULL) {
+		sb_revoked_txt = safe_strdup(db_sb_revoked_txt);
+		sb_revoked_certs = GetThumbprintEntries(sb_revoked_txt);
+	}
+	assert(sb_revoked_certs != NULL && sb_revoked_certs->count != 0);
+	if (sb_revoked_certs == NULL)
+		return FALSE;
+
+	for (i = 0; i < sb_revoked_certs->count; i++) {
+		if (memcmp(info->thumbprint, sb_revoked_certs->list[i], SHA1_HASHSIZE) == 0) {
+			uuprintf("  Found '%s' revoked certificate", info->name);
 			return TRUE;
 		}
 	}
@@ -2188,7 +2342,7 @@ static BOOL IsRevokedByCert(cert_info_t* info)
 
 BOOL IsSignedBySecureBootAuthority(uint8_t* buf, uint32_t len)
 {
-	int i;
+	uint32_t i;
 	uint8_t* cert;
 	cert_info_t info;
 
@@ -2200,8 +2354,19 @@ BOOL IsSignedBySecureBootAuthority(uint8_t* buf, uint32_t len)
 	// Secure Boot Authority is always an issuer
 	if (GetIssuerCertificateInfo(cert, &info) != 2)
 		return FALSE;
-	for (i = 0; i < ARRAYSIZE(certauth); i += SHA1_HASHSIZE) {
-		if (memcmp(info.thumbprint, &certauth[i], SHA1_HASHSIZE) == 0)
+
+	// Fall back to embedded Secure Boot thumbprints if we couldn't access remote
+	if (sb_active_certs == NULL) {
+		sb_active_txt = safe_strdup(db_sb_active_txt);
+		sb_active_certs = GetThumbprintEntries(sb_active_txt);
+	}
+	// If we still manage to get an empty list at this stage, I sure wanna know about it!
+	assert(sb_active_certs != NULL && sb_active_certs->count != 0);
+	if (sb_active_certs == NULL || sb_active_certs->count == 0)
+		return FALSE;
+
+	for (i = 0; i < sb_active_certs->count; i++) {
+		if (memcmp(info.thumbprint, sb_active_certs->list[i], SHA1_HASHSIZE) == 0)
 			return TRUE;
 	}
 	return FALSE;
@@ -2215,13 +2380,7 @@ int IsBootloaderRevoked(uint8_t* buf, uint32_t len)
 	IMAGE_NT_HEADERS32* pe_header;
 	uint8_t* cert;
 	cert_info_t info;
-	int r;
-
-	// Fall back to embedded sbat_level.txt if we couldn't access remote
-	if (sbat_entries == NULL) {
-		sbat_level_txt = safe_strdup(db_sbat_level_txt);
-		sbat_entries = GetSbatEntries(sbat_level_txt);
-	}
+	int r, revoked = 0;
 
 	if (buf == NULL || len < 0x100 || dos_header->e_magic != IMAGE_DOS_SIGNATURE)
 		return -2;
@@ -2233,37 +2392,45 @@ int IsBootloaderRevoked(uint8_t* buf, uint32_t len)
 	cert = GetPeSignatureData(buf);
 	r = GetIssuerCertificateInfo(cert, &info);
 	if (r == 0)
-		uuprintf("  (Unsigned Bootloader)");
+		uprintf("  (Unsigned Bootloader)");
 	else if (r > 0)
-		uuprintf("  Signed by: %s", info.name);
+		uprintf("  Signed by '%s'", info.name);
 
 	if (!PE256Buffer(buf, len, hash))
 		return -1;
 	// Check for UEFI DBX revocation
-	for (i = 0; i < ARRAYSIZE(pe256dbx); i += SHA256_HASHSIZE)
-		if (memcmp(hash, &pe256dbx[i], SHA256_HASHSIZE) == 0)
-			return 1;
+	if (IsRevokedByDbx(hash, buf, len))
+		revoked = 1;
 	// Check for Microsoft SSP revocation
-	for (i = 0; i < pe256ssp_size * SHA256_HASHSIZE; i += SHA256_HASHSIZE)
+	for (i = 0; revoked == 0 && i < pe256ssp_size * SHA256_HASHSIZE; i += SHA256_HASHSIZE)
 		if (memcmp(hash, &pe256ssp[i], SHA256_HASHSIZE) == 0)
-			return 2;
+			revoked = 2;
 	// Check for Linux SBAT revocation
-	if (IsRevokedBySbat(buf, len))
-		return 3;
+	if (revoked == 0 && IsRevokedBySbat(buf, len))
+		revoked =  3;
 	// Check for Microsoft SVN revocation
-	if (IsRevokedBySvn(buf, len))
-		return 4;
+	if (revoked == 0 && IsRevokedBySvn(buf, len))
+		revoked = 4;
 	// Check for UEFI DBX certificate revocation
-	if (IsRevokedByCert(&info))
-		return 5;
-	return 0;
-}
+	if (revoked == 0 && IsRevokedByCert(&info))
+		revoked = 5;
 
-void PrintRevokedBootloaderInfo(void)
-{
-	uprintf("Found %d revoked UEFI bootloaders from embedded list", sizeof(pe256dbx) / SHA256_HASHSIZE);
-	if (ParseSKUSiPolicy() && pe256ssp_size != 0)
-		uprintf("Found %d additional revoked UEFI bootloaders from this system's SKUSiPolicy.p7b", pe256ssp_size);
+	// If signed and not revoked, print the various Secure Boot "gotchas"
+	if (r > 0 && revoked == 0) {
+		if (strcmp(info.name, "Microsoft Windows Production PCA 2011") == 0) {
+			uprintf("  Note: This bootloader may fail Secure Boot validation on systems that");
+			uprintf("  have been updated to use the 'Windows UEFI CA 2023' certificate.");
+		} else if (strcmp(info.name, "Windows UEFI CA 2023") == 0) {
+			uprintf("  Note: This bootloader will fail Secure Boot validation on systems that");
+			uprintf("  have not been updated to use the latest Secure Boot certificates");
+		} else if (strcmp(info.name, "Microsoft Corporation UEFI CA 2011") == 0 ||
+			strcmp(info.name, "Microsoft UEFI CA 2023") == 0) {
+			uprintf("  Note: This bootloader may fail Secure Boot validation on *some* systems,");
+			uprintf("  unless you enable \"Microsoft 3rd-party UEFI CA\" in your 'BIOS'.");
+		}
+	}
+
+	return revoked;
 }
 
 /*
