@@ -1,7 +1,7 @@
 /*
  * Rufus: The Reliable USB Formatting Utility
  * Standard Windows function calls
- * Copyright © 2013-2024 Pete Batard <pete@akeo.ie>
+ * Copyright © 2013-2025 Pete Batard <pete@akeo.ie>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -575,13 +575,19 @@ int32_t StrArrayFind(StrArray* arr, const char* str)
 	uint32_t i;
 	if ((str == NULL) || (arr == NULL) || (arr->String == NULL))
 		return -1;
-	for (i = 0; i<arr->Index; i++) {
+	for (i = 0; i < arr->Index; i++) {
 		if (strcmp(arr->String[i], str) == 0)
 			return (int32_t)i;
 	}
 	return -1;
 }
 
+int32_t StrArrayAddUnique(StrArray* arr, const char* str, BOOL duplicate)
+{
+	int32_t i = StrArrayFind(arr, str);
+	return (i < 0) ? StrArrayAdd(arr, str, duplicate) : i;
+}
+
 void StrArrayClear(StrArray* arr)
 {
 	uint32_t i;
@@ -885,7 +891,17 @@ next_progress_line:
 		};
 	} else {
 		// TODO: Detect user cancellation here?
-		WaitForSingleObject(pi.hProcess, INFINITE);
+		switch (WaitForSingleObject(pi.hProcess, 1800000)) {
+		case WAIT_TIMEOUT:
+			uprintf("Command did not terminate within timeout duration");
+			break;
+		case WAIT_OBJECT_0:
+			uprintf("Command was terminated by user");
+			break;
+		default:
+			uprintf("Error while waiting for command to be terminated: %s", WindowsErrorString());
+			break;
+		}
 	}
 
 	if (!GetExitCodeProcess(pi.hProcess, &ret))
